#version 120
#include "/settings/lib.h"



uniform sampler2D lightmap;
uniform sampler2D texture;
uniform int isEyeInWater;
uniform float far;
uniform float frameTimeCounter;
uniform float rainStrength;
uniform vec3 skyColor;
uniform vec3 fogColor;

varying float isPlant;
varying float isOre;
varying vec2 lmcoord;
varying vec2 texcoord;
varying vec3 cPos; //Chunked Position
varying vec3 wPos; //World Position
varying vec3 sunPos; //Sun Position
varying vec3 normal;
varying vec4 position;
varying vec4 glcolor;

const float fogStart = 0.2;
const float fogEnd = 0.5;

void main() {
    vec4 diffuse = texture2D(texture, texcoord) * glcolor;
    
    vec4 color = texture2D(texture, texcoord.st);
    vec4 light = texture2D(lightmap, lmcoord.st);
    

    #ifdef UNDERWATER
    float lmy = 1.;
    if (isEyeInWater != 1) lmy = lmcoord.y;
    diffuse *= texture2D(lightmap, vec2(lmcoord.x, lmy));
    #else
    diffuse *= texture2D(lightmap, lmcoord);
    #endif

    //*-*-Data Base-*-*
    float indoor = smoothstep(0.95, 1., lmcoord.y);
    float cave = smoothstep(0.7, 1., lmcoord.y);
    float day = saturate(skyColor.r * 2.);
    float dusk = saturate((fogColor.r - 0.1) - fogColor.b);
    float mainShadow = smoothstep(0.95, 0.96, lmcoord.y);
    float sideShadow = smoothstep(0.64, 0.68, glcolor.g);

    #ifdef SHADOWS
    float shadowDepth = mix(mix(1., 0.84, day), 1., lmcoord.x * 4.2);
    diffuse.rgb *= mix(shadowDepth, 1., mainShadow);
    if (glcolor.r == glcolor.g && glcolor.g == glcolor.b) diffuse.rgb *= mix(mix(1., mix(shadowDepth, 1., saturate(dot(sunPos, normal))), mainShadow), 1., sideShadow);
    if (isPlant > 0.5) diffuse.rgb *= mix(1., mix(shadowDepth, 1., saturate(dot(sunPos, normal))), mainShadow);
    #endif
	
		// Basic shadow calculation
    float shadowFactor = max(dot(normalize(sunPos), normalize(normal)), 0.0);

    // Output the shadow intensity
    vec4 shadowColor = vec4(vec3(shadowFactor), 1.0);

    gl_FragData[0] = shadowColor;

    // Improved fog calculations
    float dist = length(position.xyz);
    float fogFactor = clamp((dist - far * fogStart) / (far * (fogEnd - fogStart)), 0.0, 1.0);
    vec3 fogColorFinal = isEyeInWater == 1 ? vec3(0.0, 0.1, 0.2) : fogColor; // Underwater fog color

    // Apply fog
    diffuse.rgb = mix(diffuse.rgb, fogColorFinal, fogFactor);

    gl_FragData[0] = diffuse;
	
	
}

